import SwiftUI
import HealthKit
import HKDatabase
import SFSafeSymbols

struct WorkoutTab: View {

    @EnvironmentObject
    var database: HealthDatabase

    @State var navigationPath: NavigationPath = .init()

    @State var workouts: [Workout] = []

    var body: some View {
        NavigationStack(path: $navigationPath) {
            VStack {
                List {
                    ForEach(workouts) { workout in
                        NavigationLink(value: workout) {
                            VStack(alignment: .leading) {
                                Text(workout.typeString)
                                    .font(.headline)
                                Text(workout.dateString)
                                    .font(.caption)
                                    .foregroundStyle(.secondary)
                            }
                        }
                    }
                }
            }
            .navigationTitle("Workouts")
            .navigationDestination(for: Workout.self) {
                WorkoutDetailView(workout: $0)
            }
            .refreshable {
                reloadAsync()
            }
            .toolbar {
                ToolbarItem {
                    NavigationLink {
                        SearchHealthStoreView()
                    } label: {
                        Image(systemSymbol: .magnifyingglass)
                    }
                }
            }.onChange(of: database.file, perform: { value in
                reload()
            })
        }
    }

    private func reload() {
        Task {
            reloadAsync()
        }
    }

    private func reloadAsync() {
        guard let store = database.store else {
            DispatchQueue.main.async {
                self.workouts = []
            }
            return
        }
        do {
            let workouts = try store.workouts()
            DispatchQueue.main.async {
                self.workouts = workouts
                print("Loaded \(workouts.count) workouts")
            }
        } catch {
            print("Failed to load workouts: \(error)")
            DispatchQueue.main.async {
                self.workouts = []
            }
        }
    }
}

#Preview {
    WorkoutTab()
        .environmentObject(HealthDatabase.mock)
}

/*
 data_series.hfd_key -> location_series_data.series_identifier

 samples.data_id = data_series.data_id
 samples.data_id = quantity_samples.data_id
 samples.data_id = objects.data_id
 samples.data_id = quantity_sample_series.data_id

 objects.provenance = data_provenances.ROWID

 data_provenances.tz_name
 data_provenances.origin_device

 Samples -> Quantity Sample Series -> Quantity Series Data
 Samples -> Data Series


 quantity_sample_series.hfd_key = quantity_series_data.series_identifier

 */
