import SwiftUI
import Collections
import HealthKit
import HKDatabase
import HealthKitExtensions
import CoreLocation

struct WorkoutDetailView: View {

    @EnvironmentObject
    var database: HealthDatabase

    private let store = HKHealthStore()

    let workout: Workout

    @State
    private var healthWorkout: HKWorkout?

    @State
    var heartRateSamplesInHealth: [HeartRate] = []

    @State
    var heartRateSamplesInDatabase: [HeartRate] = []

    @State
    var locationSamples: [CLLocation] = []

    var body: some View {
        List {
            Section("Info") {
                DetailRow("ID", value: workout.id)
                DetailRow("Total Distance", kilometer: workout.totalDistance)
                DetailRow("Duration", duration: workout.duration)
                DetailRow("Goal", value: workout.goal)
            }
            if !workout.workoutActivities.isEmpty {
                Section("Activities") {
                    ForEach(workout.workoutActivities, id: \.startDate) { activity in
                        NavigationLink(value: activity) {
                            DetailRow(activity.workoutConfiguration.activityType.description,
                                      date: activity.startDate)
                        }
                    }
                }
            }
            if !workout.workoutEvents.isEmpty {
                Section("Events") {
                    NavigationLink(value: workout.workoutEvents) {
                        DetailRow("Events", value: workout.workoutEvents.count)
                    }
                }
            }
            if !workout.metadata.isEmpty {
                Section("Metadata") {
                    NavigationLink {
                        WorkoutMetadataView(metadata: workout.metadata)
                    } label: {
                        DetailRow("Metadata", value: workout.metadata.count)
                    }
                }
            }
            Section("Heart Rate") {
                DetailRow("Samples", value: "\(heartRateSamplesInDatabase.count)")
                DetailRow("Range", value: "\(heartRateSamplesInDatabase.minimumHeartRate) - \(heartRateSamplesInDatabase.maximumHeartRate)")
            }
            if let healthWorkout {
                Section("Matching health workout") {
                    DetailRow("Duration", value: healthWorkout.duration.durationString)
                    DetailRow("Distance", kilometer: healthWorkout.distance?.doubleValue(for: .meterUnit(with: .kilo)))
                    DetailRow("Heart rate samples", value: "\(heartRateSamplesInHealth.count)")
                    DetailRow("Heart rate range", value: "\(heartRateSamplesInHealth.minimumHeartRate) - \(heartRateSamplesInHealth.maximumHeartRate)")
                }
            }


            if !locationSamples.isEmpty {
                Section("Locations") {
                    DetailRow("Count", value: "\(locationSamples.count)")
                }
            }
        }
        .navigationTitle(workout.typeString)
        .navigationDestination(for: HKWorkoutActivity.self) { activity in
            ActivityDetailView(workout: workout, activity: activity)
        }
        .navigationDestination(for: [HKWorkoutEvent].self) {
            WorkoutEventsView(events: $0)
        }
        .onAppear(perform: loadSamples)
    }

    private func loadSamples() {
        Task {
            checkPermissionsAndSearchHealth()
            do {
                guard let samples: [HeartRate] = try database.store?.samples(associatedWith: workout) else {
                    return
                }
                DispatchQueue.main.async {
                    self.heartRateSamplesInDatabase = samples
                }
                print("Loaded \(samples.count) heart rate samples from database")
            } catch {
                print("Failed to load heart rate samples from database: \(error)")
            }
        }
    }

    private func checkPermissionsAndSearchHealth() {
        Task {
            do {
                try await checkPermissionsAndFindWorkout()
            } catch {
                print("Failed to search for workout: \(error)")
            }
        }
    }

    private func checkPermissionsAndFindWorkout() async throws {

        switch  store.authorizationStatus(for: .workoutType()) {
        case .notDetermined:
            try await requestWorkoutPermission()
            try await checkPermissionsAndFindWorkout()
        case .sharingAuthorized:
            findWorkoutInHealth()
        case .sharingDenied:
            print("No permission to write workouts")
            findWorkoutInHealth()
            return
        @unknown default:
            print("Unknown permission for workouts")
            return
        }
    }

    private func requestWorkoutPermission() async throws {
        try await store.requestAuthorization(read: HKWorkout.self)
    }

    private func findWorkoutInHealth() {
        guard let activityType = workout.workoutActivities.first?.workoutConfiguration.activityType else {
            print("No activity type to find workout")
            return
        }
        let start = workout.startDate.addingTimeInterval(-60)
        let end = workout.endDate.addingTimeInterval(60)
        let workoutPredicate = HKQuery.predicateForWorkouts(with: activityType)
        let timePredicate = HKQuery.predicateForSamples(withStart: start, end: end)
        let predicate = NSCompoundPredicate(andPredicateWithSubpredicates: [workoutPredicate, timePredicate])
        let sortDescriptor = NSSortDescriptor(
            key: HKSampleSortIdentifierEndDate,
            ascending: true)

        let query = HKSampleQuery(
            sampleType: .workoutType(),
            predicate: predicate,
            limit: 0,
            sortDescriptors: [sortDescriptor]) { _, samples, error in
                if let error {
                    print("Failed to search for workout: \(error)")
                }
                guard let workout = samples?.first as? HKWorkout else {
                    print("No suitable workout found: \(samples?.count ?? 0)")
                    return
                }
                foundHealthStore(workout: workout)
            }

        store.execute(query)
    }

    private func foundHealthStore(workout: HKWorkout) {
        print("Found matching workout in health")
        DispatchQueue.main.async {
            self.healthWorkout = workout
        }
        findHealthStoreHeartRates(for: workout)
    }

    private func findHealthStoreHeartRates(for workout: HKWorkout) {
        let forWorkout = HKQuery.predicateForObjects(from: workout)
        let heartRate = HKQuantityType(.heartRate)

        let heartRateDescriptor = HKQueryDescriptor(
            sampleType: heartRate,
            predicate: forWorkout)

        let heartRateQuery = HKSampleQuery(
            queryDescriptors: [heartRateDescriptor],
            limit: HKObjectQueryNoLimit)
        { query, samples, error in
            if let error {
                print("Failed to search for heart rates: \(error)")
            }
            guard let samples else {
                print("No heart rate samples found in Health")
                return
            }
            let heartRates = samples.map { HeartRate(sample: $0) }
            processHealthStore(heartRateSamples: heartRates)
        }

        store.execute(heartRateQuery)
    }

    private func processHealthStore(heartRateSamples: [HeartRate]) {
        print("Found \(heartRateSamples.count) heart rate samples in Health")
        DispatchQueue.main.async {
            self.heartRateSamplesInHealth = heartRateSamples
        }
    }
}

#Preview {
    return NavigationStack {
        WorkoutDetailView(workout: .mock1)
            .environmentObject(HealthDatabase.mock)
    }
}

extension String: Identifiable {
    
    public var id: Self { self }
}

private extension Array where Element == HeartRate {

    var minimumHeartRate: Int {
        self.min {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }

    var maximumHeartRate: Int {
        self.max {
            $0.value < $1.value
        }?.beatsPerMinute ?? 0
    }
}

private extension HeartRate {

    var beatsPerMinute: Int {
        quantity.doubleValue(for: .count().unitDivided(by: .minute())).roundedInt
    }
}
